
    #	Create HTML documents for transit catalogue
    
    #	Input:	transits.csv
    #	Output: tr_*.html
    
    #	by John Walker
    #	http://www.fourmilab.ch/
    
    #	For details, please see: http://www.fourmilab.ch/documents/canon_transits/

    use strict;
    use warnings;
    require 'julian.pl';
    
    my $gzip = 1;   	    	    	# Set to 1 to generate Gzipped HTML files
    
    my $l;
    my %transcount;
    my $transgrand = 0;
    
    my @planets = (
    	    	    	'Mercury',
			'Venus',
			'Earth',
			'Moon',
			'Mars',
			'Jupiter',
			'Saturn',
			'Uranus',
			'Neptune',
			'Pluto'
    	    	  );
		  
    my %transfiles;
    my (%translast, %translpre);
    
    my (%htmltitle, %htmlcolhead, %htmlcoldoc);
    
    my $limitLines = 0;	    	    # Only read this many lines from transit canon (0 = all)
#$limitLines = 20000;
    my $lineno = 0;
    
    makeHTMLtitles();
    
    my $gzsuffix = $gzip ? '.gz' : '';

#    open(FI, "bzcat transits.csv.bz2 |");
    open(FI, "<transits.csv");
    while ($l = <FI>) {
    	$lineno++;
	if ($lineno eq $limitLines) {
	    last;
	}
    	$l =~ s/\s+//;
	$l =~ m/^([^\,]+),([^\,]+),([^\,]+),([^\,]+),([^\,]+),([^\,]+),([^\,]+)$/;
    	my ($transitof, $from, $start, $max, $end, $sunsubtends, $transangle) =
	    ($1, $2, $3, $4, $5, $6, $7);
	
	if ($transangle eq '.') {
	    $transangle = "0.0";    	    # Grazing transit
    	}
#print("$transitof, $from, $start, $max, $end, $sunsubtends, $transangle\n");
#ptrans(*STDOUT, "$transitof from $from  ", $start, $max, $end, $sunsubtends, $transangle, '');
    	my $sfrom = sprintf("%-8s", $from);
	my $sof = sprintf("%-8s", $transitof);
	addtrans("from_$from", $sof, $start, $max, $end, $sunsubtends, $transangle);
	addtrans("of_$transitof", $sfrom, $start, $max, $end, $sunsubtends, $transangle);
	addtrans("${transitof}_from_$from", '', $start, $max, $end, $sunsubtends, $transangle);
    	$transcount{"$transitof-$from"}++;
	$transcount{"$transitof-"}++;
    	$transcount{"-$from"}++;
	$transgrand++;
    }
    close(FI);

    #	Close all the transit event files we opened
    
    my $k;
    foreach $k (keys %transfiles) {
    	print { $transfiles{$k} } <<"EOD";
</pre>
</div>
</body>
</html>
EOD
    	close($transfiles{$k});
    }

    my ($i, $j);

if (0) {    
    for ($i = 1; $i <= $#planets; $i++) {
    	print("Transits from $planets[$i]\n");
    	for ($j = 0; $j < $i; $j++) {
	    if (!(defined $transcount{"$planets[$j]-$planets[$i]"})) {
	    	$transcount{"$planets[$j]-$planets[$i]"} = 0;
	    }
	    printf("%8s %8d\n", $planets[$j], $transcount{"$planets[$j]-$planets[$i]"});
	}
    }
}

    #	Print overall index and summary, cross-linked to transit event files

    open(IX, ">tr_index.html") || die "Cannot open trindex.html";

    print IX <<"EOD";
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">
<html version="-//W3C//DTD HTML 3.2 Final//EN">
<head>
<title>Quarter Million Year Canon of Solar System Transits</title>
<link rel="stylesheet" type="text/css"
      href="http://www.fourmilab.ch/documents/canon_transits/canon.css">
</head>
<body bgcolor="#FFFFFF">
<div class="bodycopy">
<center>

<h1>Quarter Million Year Canon of Solar System Transits</h1>

</center>

The following table gives the number of transits of objects
in the column heading as seen from objects in the row titles.
Mutual events of the Earth and Moon are excluded, as calculation of
solar and lunar eclipses is more complicated than planetary
transits.

<p>

Clicking on column headings will display a list of all transits
of that body visible from outer planets.  Clicking on a row
title displays all transits visible from that body.  Clicking
on an item within the table shows the list of transits of the
body in the column as seen from the body in the row.  Some of the
transit list files are large; the quarter million year canon includes
a total of $transcount{"Mercury-"} transits of Mercury alone!  To minimise download
time, these files are compressed with
<a href="http://www.gzip.org/" target="_blank">Gzip</a>.  Most modern
browsers automatically uncompress such files and display them
like any other Web page.  If your browser insists on saving the file
to disc, go ahead and do so, then uncompress it with
Gzip and view the resulting HTML with a "<tt>file:</tt>" URL.  If you
don't understand what this means, I can't help you.

<p>

<center>
<table border="border" bgcolor="#FFFFD0" cellpadding="5">
<tr>
<th align="left">From&nbsp;/&nbsp;Of
EOD

    my ($l1, $l2);
    
    for ($i = 1; $i <= $#planets; $i++) {
    	($l1, $l2) = ('', '');
	if ($i <= $#planets) {
	    $l1 = "<a href=\"tr_of_$planets[$i - 1].html$gzsuffix\" target=\"tr_canon_event\">";
	    $l2 = "</a>";
	}
    	print(IX "<th align=\"center\">$l1$planets[$i - 1]$l2\n");
    }
    print(IX "<th align=\"center\">Total from\n");

    for ($i = 1; $i <= $#planets; $i++) {
	$l1 = "<a href=\"tr_from_$planets[$i].html$gzsuffix\" target=\"tr_canon_event\">";
	$l2 = "</a>";
    	print(IX "<tr><th align=\"left\">$l1$planets[$i]$l2\n");
    	for ($j = 0; $j < $i; $j++) {
	    if (!(defined $transcount{"$planets[$j]-$planets[$i]"})) {
	    	$transcount{"$planets[$j]-$planets[$i]"} = '';
	    }
	    my $guzz = $transcount{"$planets[$j]-$planets[$i]"};
    	    ($l1, $l2) = ('', '');
	    my $bcol = '';
    	    if (($planets[$j] eq 'Earth') && ($planets[$i] eq 'Moon')) {
	    	$bcol = ' bgcolor="#D0D0B0"';
	    }
	    if ($guzz ne '') {
	    	$l1 = "<a href=\"tr_$planets[$j]_from_$planets[$i].html$gzsuffix\" target=\"tr_canon_event\">";
	    	$l2 = "</a>";
    	    } else {
	    	$guzz = "&nbsp;";
    	    }
	    print(IX "<td align=\"right\"$bcol>$l1$guzz$l2\n");
	}
	for ( ; $j < $#planets; $j++) {
    	    if (($planets[$i] eq 'Earth') && ($planets[$j] eq 'Moon')) {
	    	print(IX "<td bgcolor=\"#D0D0B0\">&nbsp;\n");
	    } else {
	    	print(IX "<td>&nbsp;\n");
	    }
	}
	if (defined $transcount{"-$planets[$i]"}) {
	    my $wuzz = $transcount{"-$planets[$i]"};
    	    print(IX "<td align=\"right\"><a href=\"tr_from_$planets[$i].html$gzsuffix\" target=\"tr_canon_event\">$wuzz</a>\n");
	} else {
    	    print(IX "<td align=\"right\">&nbsp;\n");
	}
    }
    
    print(IX "<tr><th align=\"left\">Total of\n");
    for ($i = 0; $i < $#planets; $i++) {
    	if (defined $transcount{"$planets[$i]-"}) {
	    my $fc = $transcount{"$planets[$i]-"};
	    print(IX "<td align=\"right\"><a href=\"tr_of_$planets[$i].html$gzsuffix\" target=\"tr_canon_event\">$fc</a>\n");
	} else {
	    print(IX "<td align=\"right\">&nbsp;\n");
	}
    }
    print(IX "<td align=\"right\">$transgrand\n");

    print IX <<"EOD";
</table>
</center>
</div>
</body>
</html>
EOD
    close(IX);

    #	Print a transit item on a designated indirect file handle
    
    sub ptrans {
    	my ($f, $prefix, $start, $max, $end, $sunsubtends, $transangle, $suffix) = @_;
	
	my ($calendar, $year, $month, $day);
	
	if (int($max + 0.5) < 2299161) {
	    $calendar = 'J';
	    ($year, $month, $day) = jd_to_julian($max);
	} else {
	    $calendar = 'G';
	    ($year, $month, $day) = jd_to_gregorian($max);
    	}
	my ($hour, $minute, $second) = jd_to_civil_time($max);

	printf($f "%s%7d-%02d-%02d %02d:%02d %s %12.2f  %12.2f  %12.2f  %.4f  %.4f%s\n",
	    $prefix,
	    $year, $month, $day, $hour, $minute, $calendar,
	    $start, $max, $end, $sunsubtends, $transangle, $suffix);
    }
    
    #	Add a transit to a specified transit event file
    
    sub addtrans {
    	my ($event, $prefix, $start, $max, $end, $sunsubtends, $transangle) = @_;
	
	#   If an event file hasn't already been opened for this event,
	#   create one and write the header.
	
	if (!(defined($transfiles{$event}))) {
	    my $fname = "tr_$event.html$gzsuffix";
print(STDERR "Opening event file $fname\n");
    	    open($transfiles{$event}, $gzip ? "| gzip >$fname" : ">$fname") || die "Cannot create event file $fname";
	    $htmlcolhead{$event} =~ s/\n$//;
    	    print { $transfiles{$event} } <<"EOD";
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">
<html version="-//W3C//DTD HTML 3.2 Final//EN">
<head>
<title>$htmltitle{$event}</title>
<link rel="stylesheet" type="text/css"
      href="http://www.fourmilab.ch/documents/canon_transits/canon.css">
<meta name="description" content="$htmltitle{$event}">
<meta name="author" content="John Walker">
<meta name="robots" content="noindex">
</head>
<body bgcolor="#FFFFFF">
<div class="bodycopy">
<center>
<h1>$htmltitle{$event}</h1>
</center>

This catalogue is an extract from the
<a href="http://www.fourmilab.ch/">Fourmilab</a>
<cite><a href="http://www.fourmilab.ch/documents/canon_transits/">Quarter
Million Year Canon of Solar System Transits</a></cite>.  Please
see that document for further information.

$htmlcoldoc{$event}
<pre>
$htmlcolhead{$event}
EOD
	}
	
	my $suffix = '';
	
	if (defined($translast{$event}) && ($start <= $translast{$event})) {
	    my $ps = $prefix . $translpre{$event};
	    if (($ps =~ m/Earth/) && ($ps =~ m/Moon/)) {
	    	$suffix = '  *';
	    } else {
	    	$suffix = '  <font color="#FF0000"><b>*</b></font>';
	    }
	}
	ptrans($transfiles{$event}, $prefix, $start, $max, $end, $sunsubtends, $transangle, $suffix);
	$translast{$event} = $end;
	$translpre{$event} = $prefix;
    }
    
    #	Create specific HTML content for event files.  Yes, we create more
    #	than we actually use.
    
    sub makeHTMLtitles {
    	for (my $i = 0; $i <= $#planets; $i++) {
	    my $p = $planets[$i];
	    
	    $htmltitle{"from_$p"} = "Transits Visible from $p";
	    $htmlcolhead{"from_$p"} = <<"EOD";
<b>Planet    Max-Transit Time  C    Start          Max          End        Sunrad  Depth   S</b>
======= ==================  = ============  ============  ============  ======  ======  =
EOD
	    $htmlcoldoc{"from_$p"} = <<"EOD";
<p>
<table align="center" border="2" bgcolor="#E0E0E0" cellpadding="3">
<tr><th align="left">Planet
    <td>Name of planet transiting the Sun.
<tr><th align="left">Max-Transit Time
    <td>Date and time of maximum transit.
<tr><th align="left">C
    <td>Calendar: <b>J</b> = Julian, <b>G</b> = Gregorian
<tr><th align="left">Start
    <td>Julian date transit begins.
<tr><th align="left">Max
    <td>Julian date of maximum transit.
<tr><th align="left">End
    <td>Julian date transit ends.
<tr><th align="left">Sunrad
    <td>Angular radius of Sun as seen from $p, degrees.
<tr><th align="left">Depth
    <td>Angular distance of <b>Planet</b> at maximum transit from centre of Sun.
<tr><th align="left">S
    <td>Is transit simultaneous with previous item in table?
</table>
<p>
EOD

	    $htmltitle{"of_$p"} = "Transits of $p";
	    $htmlcolhead{"of_$p"} = <<"EOD";
<b> From     Max-Transit Time  C    Start          Max          End        Sunrad  Depth   S</b>
======= ==================  = ============  ============  ============  ======  ======  =
EOD
	    $htmlcoldoc{"of_$p"} = <<"EOD";
<p>
<table align="center" border="2" bgcolor="#E0E0E0" cellpadding="3">
<tr><th align="left">From
    <td>Planet from which transit of $p is visible.
<tr><th align="left">Max-Transit Time
    <td>Date and time of maximum transit.
<tr><th align="left">C
    <td>Calendar: <b>J</b> = Julian, <b>G</b> = Gregorian
<tr><th align="left">Start
    <td>Julian date transit begins.
<tr><th align="left">Max
    <td>Julian date of maximum transit.
<tr><th align="left">End
    <td>Julian date transit ends.
<tr><th align="left">Sunrad
    <td>Angular radius of Sun as seen from "<b>From</b>" planet, degrees.
<tr><th align="left">Depth
    <td>Angular distance of $p at maximum transit from centre of Sun.
<tr><th align="left">S
    <td>Is transit simultaneous with previous item in table?
</table>
<p>
EOD
	    for (my $j = 1; $j <= $#planets; $j++) {
	    	my $q = $planets[$j];
	    	$htmltitle{"${p}_from_$q"} = "Transits of $p from $q";
		$htmlcolhead{"${p}_from_$q"} = <<"EOD";
<b>  Max-Transit Time  C   Start           Max          End        Sunrad  Depth</b>
==================  = ============  ============  ============  ======  ====== 
EOD
	    $htmlcoldoc{"${p}_from_$q"} = <<"EOD";
<p>
<table align="center" border="2" bgcolor="#E0E0E0" cellpadding="3">
<tr><th align="left">Max-Transit Time
    <td>Date and time of maximum transit.
<tr><th align="left">C
    <td>Calendar: <b>J</b> = Julian, <b>G</b> = Gregorian
<tr><th align="left">Start
    <td>Julian date transit begins.
<tr><th align="left">Max
    <td>Julian date of maximum transit.
<tr><th align="left">End
    <td>Julian date transit ends.
<tr><th align="left">Sunrad
    <td>Angular radius of Sun as seen from $q, degrees.
<tr><th align="left">Depth
    <td>Angular distance of $p at maximum transit from centre of Sun.
</table>
<p>
EOD
    	    }
    	}
    }
