/*

    	Extract a range of numerical integration data in CSV format
	
	         by John Walker
	    http://www.fourmilab.ch/
    
    For details, please see: http://www.fourmilab.ch/documents/canon_transits/
	
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "readout.h"
#include "absbody.h"
#include "linalg.h"
#include "julian.h"
#include "mathutil.h"

#define ELEMENTS(array) (sizeof(array)/sizeof((array)[0]))

int main(int argc, char *argv[0])
{
    FILE *fi;
    abs_step st;
    double jdmin, jdmax;
    int i;

    if (argc != 4) {
    	printf("usage: extract input_file JD_min JD_max\n");
	return 2;
    }
    
    if ((fi = fopen(argv[1], "rb")) == NULL) {
    	printf("Cannot open input file %s\n", argv[1]);
	return 1;
    }
    jdmin = atof(argv[2]);
    jdmax = atof(argv[3]);
    if (jdmin == 0) {
    	printf("Bad JD_min specification.\n");
	exit(1);
    }
    if (jdmax == 0) {
    	printf("Bad JD_max specification.\n");
	exit(1);
    }
    if (jdmin >= jdmax) {
    	printf("JD_max is greater than JD_min.\n");
	exit(1);
    }
    
    while (readabs(fi, &st) == 0) {
    	int yy, mon, dd, hh, mm, ss;

    	if (st.jd >= jdmin) {
	    if (st.jd >= jdmax) {
	    	break;
	    }
	    jd_to_gregorian(st.jd, &yy, &mon, &dd);
	    jd_to_civil_time(st.jd, &hh, &mm, &ss);
	    printf("JD,%.4f,%d,%d,%d,%d,%d,%d\n", st.jd, yy, mon, dd, hh, mm, ss);
	    for (i = 0; i < ELEMENTS(st.obj); i++) {
	    	printf("%s,%.16g,%.16g,%.16g",
		    abs_objname[i],
		    st.obj[i].pos[X], st.obj[i].pos[Y], st.obj[i].pos[Z]);
		/* Readabs doesn't provide velocities for Earth and Moon. */
		if (i < Earth) {
	    	    printf(",%.16g,%.16g,%.16g",
		    	st.obj[i].vel[X], st.obj[i].vel[Y], st.obj[i].vel[Z]);
		}
    	    	printf("\n");
	    }
	    printf("END\n");
	}
    }

    fclose(fi);
    
    return 0;
}
