
    #	Extract a range of Julian days from output of transits.c
    
    #	Input:	all_fine.txt.gz
    #	Output: Standard output
    
    #	by John Walker
    #	http://www.fourmilab.ch/
    
    #	For details, please see: http://www.fourmilab.ch/documents/canon_transits/

    use strict;
    use warnings;

    my $l;
    
    if ($#ARGV < 1) {
    	print(STDERR "Usage: perl extract_canon.pl jd_start jd_end\n");
	exit(2);
    }
    
    my ($start, $end) = ($ARGV[0], $ARGV[1]);
    
    if ($start > $end) {
    	print(STDERR "Start date is greater than end date.\n");
	exit(2);
    }
    
    my ($Pof, $Pfrom, $Pjd);

    open(FI, "zcat ../Integration_Results/all_fine.txt.gz |") ||
    	die("Cannot open pipe from ../Integration_Results/all_fine.txt.gz");
    while ($l = <FI>) {
	if (!parse_transit($l)) {
    	    next;
	}
	if ($Pjd > $end) {
	    last;
	}
	if ($Pjd >= $start) {
	    print($l);
	}
    }    
    close(FI);

    #	Parse a transit event record into global field variables
    
    sub parse_transit {
	my ($l) = @_;
	
    	$l =~ s/\s+$//;
	if ($l =~ m/^Transit of (\w+)\s+from (\w+)\s+(\S+)\s+(.*)$/) {
	    ($Pof, $Pfrom, $Pjd) = ($1, $2, $3);
	    return 1;
	}
	return 0;
    }
