/*

    	Read output file from ssystem
	
	         by John Walker
	    http://www.fourmilab.ch/
    
    For details, please see: http://www.fourmilab.ch/documents/canon_transits/
	
*/

#include <stdio.h>
#include <math.h>

/*  The following two files are included from the DE118i-2
    build directory, which is assumed to be a peer of this
    directory in their common parent.  Including the precision
    and data structures files from that directory makes sure
    we share the same assumptions about the format written
    by the integrator.  Note that if you've compiled DE118i-2
    with options which affect data structures (for example,
    alignment of 80 bit floating point quantities on 16 byte
    boundaries, packing of structures, etc.) you must use the
    same compile options for this file and those linked with
    it.  */
#include "../DE118i-2/prec.h"
#include "../DE118i-2/ssystem.h"

#include "readout.h"

/*  Determine the floating point type found in the output file.
    This is double unless LDOUBLE is defined and OUTDOUBLE is not,
    in which case it's whatever prec.h defined DOUBLE to be, namely
    long double.  */

#if LDOUBLE
#if OUTDOUBLE
#undef DOUBLE
#define DOUBLE double
#endif
#endif
    
/*  Object names  */

const char *ss_objname[Sun + 1] = {
     "Librations", "Mercury", "Venus", "EMB",
     "Mars", "Jupiter", "Saturn", "Uranus",
     "Neptune", "Pluto", "Moon_Vec", "Sun"
};

/*  READOUT  --  Read next state vector record from file fi into
    	    	 ss_step s.  Returns 0 if the record was read
		 successfully or EOF if end of file was encountered
		 (including a truncated final record).  */

int readout(FILE *fi, ss_step *s)
{
    int i, j;
    DOUBLE jd, statevec[6 * (ISUN + 1)];
 
    /*  Sanity check conformance between DE118i configuration and
	the interface we define for callers of readout.  */

#ifndef NDEBUG
    if (ISUN != Sun) { 
    	printf("Error--ISUN (%d) in ssystem.h does not equal Sun (%d) in readout.h\n", ISUN, Sun);
    }
#endif
    
    if (fread(&jd, sizeof(DOUBLE), 1, fi) > 0) {
    	s->jd = jd;
	if (fread(statevec, sizeof statevec, 1, fi) > 0) {
 	    for (i = 0; i < (ISUN + 1); i++) {
    		for (j = 0; j <= 2; j++) {
	    	    s->obj[i].pos[j] = statevec[(6 * i) + (j * 2) + 1];
	    	    s->obj[i].vel[j] = statevec[(6 * i) + (j * 2) + 0];
		}
	    }
	}
	return 0;
    }
    
    return EOF;
}
