

    #	Prepare CSV simultaneous transit database from
    #	master transit CSV database.
    
    #	Input:	transits.csv  (May be overridden by command line argument 1)
    #	Output: simultaneous.csv
    
    #	by John Walker
    #	http://www.fourmilab.ch/
    
    #	For details, please see: http://www.fourmilab.ch/documents/canon_transits/
    
    use strict;
    use warnings;
    
    my $includeEarthMoon = 0;	# Include Earth/Moon only as double transits
    my $trace = 0;  	    	# Detailed trace of decision making if nonzero

    #	Load the transits database into memory
    
    my $ifn = "transits.csv";	    # Default input file
    if ($#ARGV >= 0) {
    	$ifn = $ARGV[0];    	    # Allow override from command line
    }
    
    open(FI, "<$ifn") || die("Cannot open $ifn");    

    my @transits;
    my $l;
    print(STDERR "Loading transits into memory.\n");
    while ($l = <FI>) {
    	$l =~ s/\s+//;
	push(@transits, $l);
    }
    close(FI);
    
    my @simultaneous;
    
    my $lineno = 0;
    my $lstart = -1E100;
    
    my @inProgress; 	    	#   Transits in progress
    
    print(STDERR "Searching for simultaneous transits.\n");
    for (my $j = 0; $j <= $#transits; $j++) {
    	$lineno++;
    	$l = $transits[$j];
	$l =~ m/^([^\,]+),([^\,]+),([^\,]+),([^\,]+),([^\,]+),([^\,]+),([^\,]+)$/ || die("Parse error on $l");
    	my ($transitof, $from, $start, $max, $end, $sunsubtends, $transangle) =
	    ($1, $2, $3, $4, $5, $6, $7);
	    
	die("Last start $lstart > this start $start") if ($lstart > $start);
	$lstart = $start;
	    
	#   Walk through the table and see if this transit overlaps any
	#   active transit.  If so, create a new mutual transit for
	#   the overlap with a multiplicity one greater than the
	#   existing item.
	
	my $ipn = $#inProgress;
print("Testing: items 0-$ipn  against $l\n") if $trace;
	for (my $i = 0; $i <= $ipn; $i++) {
	    my ($o, $mstart, $mend) = overlaps($inProgress[$i], $from, $start, $end);
print(" against: $inProgress[$i]{from} $inProgress[$i]{start}-$inProgress[$i]{end} $inProgress[$i]{multiplicity}\n") if $trace;
	    if ($o) {
#	    	if (!(($inProgress[$i]{transrec}[0] =~ m/^(Earth|Moon)/) &&
#		      ($l =~ m/^(Earth|Moon)/))) {
print("  Overlap: \n") if $trace;
print("    " . $inProgress[$i]{transrec}[0] . "\n") if $trace;
    	    	    my @tr;
		    push(@tr, @{$inProgress[$i]{transrec}});
		    push(@tr, $l);
		    push(@inProgress,
			{
	    		    multiplicity => $inProgress[$i]{multiplicity} + 1,
			    from => $from,
			    start => $mstart,
			    end => $mend,
			    transrec =>  [ @tr ]
			}
		    );
#		}
	    }
else {
print("  No overlap: \n") if $trace;
print("    " . $inProgress[$i]{transrec}[0] . "\n") if $trace;
}
	}
	
	
	#   Purge any items from the table whose end times are less
	#   than the start time of this element.  Since we've sorted
	#   by start time, we're guaranteed no subsequent item can
	#   overlap the one we remove.  If it has a multiplicity
	#   greater than 1, record it as a mutual event.
	
	for (my $i = 0; $i <= $#inProgress; ) {
	    if ($inProgress[$i]{end} < $start) {
	    	if ($inProgress[$i]{multiplicity} > 1) {
if ($trace) {
print("\nMultiplicity: " . $inProgress[$i]{multiplicity} . "\n");
for (my $k = 0; $k < $inProgress[$i]{multiplicity}; $k++) {
    print("  " . $inProgress[$i]{transrec}[$k] . "\n");
}
}
    	    	    #	Here's were we decide whether to exclude a double transit which
		    #	consists solely of the Earth and Moon.  These are quite common,
		    #	and the inflate the table of multiple transits enormously if you
		    #	include them.
    	    	    if ($includeEarthMoon ||
		    	($inProgress[$i]{multiplicity} > 2) ||
		    	    (!((($inProgress[$i]{transrec}[0] =~ m/^Earth/) && ($inProgress[$i]{transrec}[1] =~ m/^Moon/)) ||
			       (($inProgress[$i]{transrec}[1] =~ m/^Earth/) && ($inProgress[$i]{transrec}[0] =~ m/^Moon/))))
		       ) {
    	    		my $csvr = "$inProgress[$i]{from},$inProgress[$i]{start},$inProgress[$i]{end},$inProgress[$i]{multiplicity}";
			for (my $k = 0; $k < $inProgress[$i]{multiplicity}; $k++) {
		    	    my $r = $inProgress[$i]{transrec}[$k];
			    $r =~ m/^([^\,]+),([^\,]+),([^\,]+),([^\,]+),([^\,]+),([^\,]+),([^\,]+)$/ || die("Parse error on $l");
    			    my ($transitof, $from, $start, $max, $end, $sunsubtends, $transangle) =
				($1, $2, $3, $4, $5, $6, $7);
			    $csvr .= ",$transitof,$start,$max,$end";
			}
			$csvr .= "\n";
			push(@simultaneous, $csvr);    
		    }
else {
    print("Bounced Earth/Moon double transit.\n") if $trace;
}
    	    	}
		splice(@inProgress, $i, 1);
	    } else {
	    	$i++;
	    }
    	}
	    
	#   Add this transit to the array of in-progress transits
	#   with a multiplicity of 1.
	
	push(@inProgress,
	    {
	    	multiplicity => 1,
		from => $from,
		start => $start,
		end => $end,
		transrec => [ $l ]
	    }
	);
    }
    
    my $n = $#inProgress;
    print(STDERR "$n transits in progress at end.\n");
    
#    print(STDERR "Sorting simultaneous events by start time.\n");    
#    my @ssimul = sort bysimstart @simultaneous;

    print(STDERR "Writing simultaneous.csv.\n"); 
    open(FO, ">simultaneous.csv");
    my $lastst = -1E100;
    for (my $i = 0; $i <= $#simultaneous; $i++) {
    	$simultaneous[$i] =~ m/^\w+,([^\,]+),/;
	die("Simultaneous records not sorted by start time: $1 < $lastst") if ($1 < $lastst);
	$lastst = $1;
    	print(FO $simultaneous[$i]);
    }
    close(FO);
    
    #	Comparison function to sort transit records by start time
    
    sub bystart {
	$a =~ m/^[^\,]+,[^\,]+,([^\,]+),/;
    	my $astart = $1;
	$b =~ m/^[^\,]+,[^\,]+,([^\,]+),/;
    	my $bstart = $1;
	return $astart <=> $bstart;
    }
    
    #	Comparison function to sort simultaneous records by start time
    
    sub bysimstart {
	$a =~ m/^[^\,]+,([^\,]+),/;
    	my $astart = $1;
	$b =~ m/^[^\,]+,([^\,]+),/;
    	my $bstart = $1;
	return $astart <=> $bstart;
    }
    
    #	Determine whether a transit item overlaps with
    #	an active item in the table.  Returns a Boolean
    #	indicating whether there's an overlap and, if so,
    #	the start and end times of the period of overlap.
    
    sub overlaps {
    	my ($item, $from, $start, $end) = @_;
	
	if ($item->{from} ne $from) {
	    return (0, 1, 0);
	}
	my $mutstart = max($item->{start}, $start);
	my $mutend = min($item->{end}, $end);
	return (($mutstart <= $mutend), $mutstart, $mutend);
    }
    
    #	Return least of arguments
    
    sub min {
    	my $v = shift;
	
	my $n;
	while ($n = shift) {
	    if ($n < $v) {
	    	$v = $n;
	    }
	}
	return $v;
    }
    
    #	Return greatest of arguments
    
    sub max {
    	my $v = shift;
	
	my $n;
	while ($n = shift) {
	    if ($n > $v) {
	    	$v = $n;
	    }
	}
	return $v;
    }
